DAT programName         byte "BluetoothBridgeHC_05_150822c", 0
CON
{{
  By Duane Degn
  August 22, 2015

  
}}
{
  ******* Private Notes *******
  150822a Convert to work with both 5MHz crystals and 10MHz crystals.
  22a Make setting baud on HC-05 easier.
}  
CON

  _clkmode = xtal1 + pll8x                           
  _xinfreq = 10_000_000

  CLK_FREQ = 80_000_000
  MILLISECOND = CLK_FREQ / 1_000

  '' I/O pins
  BLUETOOTH_KEY = 14
  BLUETOOTH_TX = 15
  BLUETOOTH_RX = 16

  PST_BAUD = 115_200           
            
  CHANGE_BAUD_CHARACTER = "&"   ' Use this character to enter new baud for Bluetooth communication.
  SET_KEY_LOW = "\"
  SET_KEY_HIGH = "|"
  BT_SERIAL_MODE = 0 '%1000

  AT_ATTEMPTS = 4
  
VAR

  long btBaud
  
  byte btBuffer[Bt#BUFFER_LENGTH + 1]
  
OBJ

  Pst : "Parallax Serial Terminal"
  Bt : "Parallax Serial Terminal"
  Format : "StrFmt"
  Crystal : "CrystalSet"
  
PUB Start

  Crystal.Set
  
  outa[BLUETOOTH_KEY] := 0
  dira[BLUETOOTH_KEY] := 1

  Pst.Start(PST_BAUD)

  repeat
    Pst.Str(string(11, 13, "Press any key to begin."))
    result := Pst.RxCount
    waitcnt(clkfreq / 4 + cnt)
  until result

  Pst.RxFlush
  Pst.Clear
  Pst.Home
  Pst.Str(string(11, 13, "programName = "))
  Pst.Str(@programName)
  Pst.Str(string(11, 13, "HC-05 Test Program."))
  
  btBaud := SearchForBt

  ifnot btBaud
    Pst.Str(string(11, 13, "The program was not able to determine the baud setting for the Bluetooth module."))
    Pst.Str(string(11, 13, "Enter new Bluetooth baud: "))
    btBaud := Pst.DecIn
    Pst.Str(string(11, 13, "New Bluetooth baud = "))
    Pst.Dec(btBaud)
    Bt.StartRxTx(BLUETOOTH_RX, BLUETOOTH_TX, BT_SERIAL_MODE, btBaud) 
   
  BridgeInstuctions

  BridgeBt

PUB BridgeBt | inputCharacter, numberOfCharactersInBuffer, btIndex

  btIndex := 0
  
  repeat
    '' This first section of the loop checks for input from the terminal.
    '' If input is received it is passed on to the Bt only after
    '' a chariage return is received.
    '' The BT message is terminated with a both a carriage return
    '' and a line feed.

    numberOfCharactersInBuffer := Pst.RxCount
    if numberOfCharactersInBuffer
      repeat numberOfCharactersInBuffer
        inputCharacter := Pst.CharIn
        btBuffer[btIndex++] := inputCharacter
        case inputCharacter
          $0D: ' Add a line feed character when a carriage return is received.
            btBuffer[btIndex++] := $0A
            btBuffer[btIndex] := 0
            Bt.Str(@btBuffer)
            Pst.Str(string(11, 13, "Sent following string to Bluetooth: "))
            SafeStr(@btBuffer, btIndex)
            btIndex := 0    

          CHANGE_BAUD_CHARACTER:
            Pst.Str(string(11, 13, "Enter new Bluetooth baud: "))
            btBaud := Pst.DecIn
            Pst.Str(string(11, 13, "New Bluetooth baud = "))
            Pst.Dec(btBaud)
            Bt.StartRxTx(BLUETOOTH_RX, BLUETOOTH_TX, BT_SERIAL_MODE, btBaud)
            BridgeInstuctions
          SET_KEY_LOW:
            outa[BLUETOOTH_KEY] := 0
            BridgeInstuctions
            btIndex := 0
          SET_KEY_HIGH:
            outa[BLUETOOTH_KEY] := 1
            BridgeProgramInstuctions
            btIndex := 0
                   
    '' This last section of the loop checks for input from the Bt.
    '' If input is received it is passed on to the terminal.
    '' Non-printable ASCII characters will be displayed as their
    '' hexadecimal value.
    
    numberOfCharactersInBuffer := Bt.RxCount
    if numberOfCharactersInBuffer
      repeat numberOfCharactersInBuffer
        inputCharacter := Bt.CharIn
        SafeTx(inputCharacter, true)

PUB BridgeInstuctions 

  Pst.Str(string(11, 13, 11, 13, "                       Instructions"))
  Pst.Str(string(11, 13, "Whatever you type (with a few exceptions) will be passed to the Bluetooth module."))
  Pst.Str(string(11, 13, "The text is only transmitted to the module when a carriage return is detected."))
  Pst.Str(string(11, 13, "A line feed character is added to the message.", 11, 13))
  Pst.Str(string(11, 13, "The HC-05 can be placed into programming mode by setting the KEY pin high."))
  Pst.Str(string(11, 13, "To set the KEY pin high press the ", 34))
  Pst.Char(SET_KEY_HIGH)
  Pst.Str(string(34, " character (without pressing enter)."))
  Pst.Str(string(11, 13, "To set the KEY pin low press the ", 34))
  Pst.Char(SET_KEY_LOW)
  Pst.Str(string(34, " character."))
  Pst.Str(string(11, 13, "If the baud is changed with an AT command, then you'll need to change the baud"))
  Pst.Str(string(11, 13, "used by the Propeller by pressing the ", 34))
  Pst.Char(CHANGE_BAUD_CHARACTER)
  Pst.Str(string(34, " key."))
  Pst.Str(string(11, 13, "After commands have been issued, set the KEY pin low to use the module as normal.", 11, 13, 11, 13))
   
PUB BridgeProgramInstuctions 

  Pst.Str(string(11, 13, 11, 13, "                   You have entered programing mode.", 11, 13))
  Pst.Str(string(11, 13, "Here are a list of some of the HC-05 AT commands."))
  Pst.Str(string(11, 13, "AT+VERSION (To read the firmware version of the module. Read only.)"))
  Pst.Str(string(11, 13, "My module returned the value ", 34, "2.0+20100601", 34, "."))
  Pst.Str(string(11, 13, "AT+RESET (To reset the module. This command can be useful to exit command mode.)"))
  Pst.Str(string(11, 13, 34, "AT+RESET", 34, " does strange things to the module's baud. The baud while in"))
  Pst.Str(string(11, 13, "command mode may be changed to 38400 but this is not consistent. It depends if"))
  Pst.Str(string(11, 13, "module is connected to another device."))
  Pst.Str(string(11, 13, "AT+DEFAULT (To place the module in its default state. I think this will change the baud  "))
  Pst.Str(string(11, 13, "and other settings back the original values."))
  Pst.Str(string(11, 13, "AT+ADDR (To read the module's address. read only.)"))
  Pst.Str(string(11, 13, "AT+CMODE (To read the module's pairing mode. 1 = remember last paired address.)"))
  Pst.Str(string(11, 13, "AT+ROLE (To read the module's master/slave state. 0 = slave, 1 = master.)"))
  Pst.Str(string(11, 13, "AT+ROLE=0 (To set the module as a slave device.)"))
  Pst.Str(string(11, 13, "AT+ROLE=1 (To set the module as a master device.)"))
  Pst.Str(string(11, 13, "AT+PSWD (To read the module's four digit password.)"))
  Pst.Str(string(11, 13, "AT+PSWD=1234 (To set the module's four digit password (", 34, "1234", 34, " used in "))
  Pst.Str(string("this example).)"))
  Pst.Str(string(11, 13, "AT+NAME (To read the module's name.)"))
  Pst.Str(string(11, 13, "The module's name may be up to 31 characters long. Alpha numberic characters are allowed."))
  Pst.Str(string(11, 13, "Both upper and lower case letters may be used. Underscores are allowed."))
  Pst.Str(string(11, 13, "AT+UART (To read the module's baud.)"))
  Pst.Str(string(11, 13, "AT+UART=115200,0,0 (To set the module's baud(", 34, "115200", 34, " used in "))
  Pst.Str(string("this example). "))
  Pst.Str(string(11, 13, "The two zeros indicate stop bit and parity settings."))
  Pst.Str(string(11, 13, "The following baud settings are allowed: 4800, 9600, 19200, 38400, 57600, 115200"))
  Pst.Str(string(11, 13, "23400, 460800, 921600, 1382400"))
  Pst.Str(string(11, 13, "The Propeller will likely have trouble communicating with setting above 115200."))
  Pst.Str(string(11, 13, "The new baud setting will not take effect until the module is turned back on."))
  Pst.Str(string(11, 13, "AT+MRAD? (To read the most recently used authenticated device.)"))
  Pst.Str(string(11, 13, "AT+STATE? (To read the work status of the module.)"))
   
  Pst.Str(string(11, 13, 11, 13, "After commands have been issued, set the KEY pin low to use the module as normal."))
  Pst.Str(string(11, 13, 11, 13, "It's a good idea to type a couple ", 34, "AT", 34, " commands in order to"))
  Pst.Str(string(11, 13, "make sure the HC-05 is ready."))
   
PUB SearchForBt | localIndex, testBaud 

  Pst.Str(string(11, 13, "Searching for present Bluetooth baud."))

  repeat localIndex from 0 to MAX_GUESS_INDEX
    testBaud := baudGuess[localIndex]
    Pst.Str(string(11, 13, "Attempt # "))
    Pst.Dec(localIndex + 1)
    Pst.Str(string(" of "))
    Pst.Dec(MAX_GUESS_INDEX + 1)
    Pst.Str(string(". Using baud  "))
    Pst.Dec(testBaud)
    Pst.Char(".")
    Bt.StartRxTx(BLUETOOTH_RX, BLUETOOTH_TX, BT_SERIAL_MODE, testBaud) 
    Pst.Str(string(11, 13, "Setting P"))
    Pst.Dec(BLUETOOTH_KEY)
    Pst.Str(string(" high to use programming mode."))
    outa[BLUETOOTH_KEY] := 1
    Pst.Str(string(11, 13, "Sending ", 34, "AT", 34, " commands and checking for ", 34, "OK", 34, " in return."))
    result := AnyoneThere(AT_ATTEMPTS)
    Pst.Str(string(11, 13, "Setting P"))
    Pst.Dec(BLUETOOTH_KEY)
    Pst.Str(string(" low to exit programming mode."))
    outa[BLUETOOTH_KEY] := 0
    if result
      quit
      
  if result
    Pst.Str(string(11, 13, "The Bluetooth module was found. It uses a baud setting of "))
    Pst.Dec(testBaud)
    Pst.Char(".")
    result := testBaud
     
PRI AnyoneThere(attempts) | count

  count := 0
  
  repeat attempts
    count++
    Pst.Str(string(11, 13, "Sending ", 34, "AT", 34, " command # "))
    Pst.Dec(count)
    Pst.Str(string(" of "))
    Pst.Dec(attempts)
    Pst.Char(".")
    Pst.Str(string(11, 13, "AT"))
    Bt.str(string("AT", 13, 10))
    result := CheckForTwoCharacters("O", "K", 250)
    if result
      Pst.str(string(11, 13, "Bluetooth found."))
      quit
    else
      Pst.str(string(11, 13, "No Bluetooth yet."))
      
PRI CheckForTwoCharacters(charFirst, charLast, timeout) | startTime, previousCharacter

  previousCharacter := 0
  timeout *= MILLISECOND

  Pst.str(string(11, 13, "Watching for the characters ", 34))
  Pst.Char(charFirst)
  Pst.Char(charLast)
  Pst.str(string(34, " from the Bluetooth module."))
        
  startTime := cnt
  repeat
    result := Bt.RxCount

    if result
      Pst.str(string(11, 13, "BT character received = "))
      result := Bt.CharIn
      SafeTx(result, false)
      if previousCharacter == charFirst and result == charLast
        Pst.str(string(11, 13, "The characters ", 34))
        Pst.Char(charFirst)
        Pst.Char(charLast)
        Pst.str(string(34, " have been received."))
        Pst.str(string(11, 13, "Flush extra characters."))
        result := Bt.RxCount
        repeat while result
          result := Bt.CharIn
          SafeTx(result, false)
          result := Bt.RxCount  
        result := true
        return
      else
        previousCharacter := result
      
  until cnt - startTime > timeout
  
  
  Pst.str(string(11, 13, "Characters not found."))
  result := 0

PRI SafeStr(localPtr, localSize)

  repeat localSize
    SafeTx(byte[localPtr++], true)
    
PRI SafeTx(localCharacter, executeCarriageReturnFlag)

  if localCharacter => 32 and localCharacter =< "~"
    Pst.Char(localCharacter)
  elseif localCharacter == 0 ' this may need to be changed if monitoring raw data
                             ' "Parallax Serial Terminal" doesn't catch framing errors
                             ' so without this "elseif" line you can end up with a
                             ' bunch of zeros if a line is inactive. 
    return
  else
    Pst.Char("<") 
    Pst.Char("$")
    Pst.Hex(localCharacter, 2)
    Pst.Char(">")

  if localCharacter == $0D and executeCarriageReturnFlag
    Pst.Char($0D)            ' The hex value of carriage return characters will be displayed
                             ' and the carriage return will also be passed to the terminal.
                             ' This should improve the readablity of the output.

DAT

baudGuess     long 9600, 115200, 38400, 1200, 2400, 19200

CON

  MAX_GUESS_INDEX = 5                        